/*
 * vkontakte.c - interface functions for driver Vkontakte of MSA program.
 * This file is part of MSA program.
 *
 * Copyright (C) 2009 - Sergey Zakharov
 *
 * MSA program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MSA program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MSA program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 */
#ifndef _MSA_DRIVERS_VKONTAKTE_H
#define _MSA_DRIVERS_VKONTAKTE_H

#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <gconf/gconf-client.h>
#include "../mysocials_driver.h"

//settings
#define STATUS "active"
#define LOGIN "login"
#define PASSWORD "password"
#define AUTOUPDATE "autoupdate"
#define REMIXMID "remixmid"
#define REMIXSID "remixsid"

#define DRIVER_ID "vkontakte"
#define DRIVER_NAME "В Контакте"
#define G_LOG_LEVEL_ERROR
#define PATH "/apps/msa/vkontakte/"

#define VK_PIC "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAYAAACqaXHeAAAABHNCSVQICAgIfAhkiAAAAAlwSFlz\
AAALEgAACxIB0t1+/AAAABx0RVh0U29mdHdhcmUAQWRvYmUgRmlyZXdvcmtzIENTNAay06AAAAet\
SURBVHic5Zt7bBTXFcZ/M7uzXs/G9gqveRkHQ1MQJhaQRDRNUoiKFEpEVJJILWpSTPmnKY1SVWqp\
oqqK2qpJiRCREKpaVLWEECU0UqsooLapiPoI0MhAQ6IQzJvF+MFiYxu8j9m5c/rHru212YU1M2sb\
+KRrrz13Pn3z7T3nnrkzVxMR7mTo4y1gvOEH0DStYIfn/9QzS3z6Mt3nW6j5fAsARETzGWUPj5HG\
oqDSqX2apgmAKHXEUepjTTl7t34jfKbQOSKCJiL5DNDWvHlpmT8YeslvBB6pCIIZAMM31MEMFDZt\
PBC3hkI5rSBuwZUk2GnrQzvZ//Mdz0T2AsPivZAB+so/dP8oaFZtnFIJk0wN2wHbASfndNsp7QWN\
Fv6cYNa1zN9+HbrjQmcfJOO9P9m9btImYFB5PgO0h15rfa4qMvU3s6vBp2vDLvpWhK6BcoTTXdB7\
qWP9/h/O+C3ZkTDSAG3+L88sD0+v+2vDFPDrE2uIu4XtCEc7oaft/IrPfjbr74CISCYJZhHwVUY2\
11UNnXC7oa4KrlyNbAYWASlg0ABt7ouHl4fM4DzTEJQaN40lhWlAyAzOm/vi4eUtr9z3HiADBhj+\
ysiT4TIH5dxeQ38kwmVCX2XkSeBvgDVgQJnmDzaahqAmWHb3GqYhaP5gI1BGjgEBPVhxf8CfyZi3\
MwJ+0IMV9wMBGMoBfkcplOMrfOZtBCeT5PyDPwAdR2Hbd8itgaMgex80OA0qR3A8GP4tJwuW3kUj\
aIaYFK6gwix3zZUPuWE+aICIoFzeGj+3IMnjz853xXG+K8F/jnWx7b+XaGmzSVlpZtXXYZYHXfHm\
QkSoWLymAmgfqASnzX21q+0LNYYr4j1rK7xRmMXZWJx3D3bwx30xJs+cg2H4b3xSETgVS9OyobpW\
RNqGGEUm3AxQX2PygxWzWdpQzWO/bqZx4X3emJAZ6QI5CyIiDsoRV61UWDiziuObltJ67pRrjcoR\
RIaKneE5wJm4VVA4ZLDxqRn84oPLhMNVrrhylwGHG6AmVgiMxKoHprGz+RgxVemKp4ABDkpKMwIO\
X0jzwru9N+x3T7WPL0b8rFscYlpF/pqk6cFqNh5wp/M6IVC6EVAMd0vMpiVmc7DV4i9N1Xn7fGVO\
mJf39bjSkn8EOIJTohAQYVTcbT2K3UeTrGy4du4PhwzXOiV/IeSULAkKo0+w0e4kkL/4cauzcAiU\
KgkKo+bWUv1A+Jr/t1y0XevMGwKI4JRqBIySuzV6jhVNC/Mea44m3essNAs4JZoFLMuis6OtqL6X\
u7vZtq6B+hoz7/EdzX2udY55CNRW+fjew8XN3U1L5he8+Ff+cZnWy7ZrPYULoRKFQH2NyUtPz3XF\
8ecjV9n+0Y1riWJQsBByJuiC6I/fOs5bR1KUmyFP+MalELpZnI3Fube2nDmnLnHwQg+RKdNdcxYo\
hByUnfdB6biivsakvsakaUkdr//7POvfOEnN1Nqb5hOR/IUQIqTtiTcCctG0pI4q08/a7VHMu9zd\
EA1gWAiUCvtPxXl629mi+jbWBvnp1yazdM5deY+vemAaTx2OseeEQtfdL+IOWxBx2wpBkKI5PmmN\
883fn+WFXa0F+TY/M48yiXuiNccAcd2uh9Fy7Tp4mX2n+vNyhUMG33pwsidaJ6wBIsLbzd0F+Rpn\
hDzROqwOKBlEboo/2pUqeKw+Uu6J5mGzQElxE/xWMu4550iMySwgN8GvlOLxxvyrQgDR7pQnmscm\
BCg+BJRS2FaKKabNE4sKP2X6tLXf2xCQEpbBtpWiJ9ZeVN+ZkXKWLqrmtW/fSziU/0nVlaRi79Fe\
TzSPSQg82hBBvfmEZ3xb97bTl1S3Ugh4h8/bE7y+/yIdvZYnfGMyArzEy7uj2WnVG723jAF9CcX6\
nSdoPn2Vdo++ffA4BD670M/8Wm8WLQbQl7DZvq+T7R92cKIz4Sk3eFwIrdryCe98v5GFd+e/kysW\
B072cq4rxaet/fzrWA9Hzl91ra0QPB0B7T0Wj/zqkGuescQtkwNKhUEDHEehaXfGW2LiXLsoKmIl\
OzGCU8ZH0thC0olOsq/IDBigJJ3swF92pxjQAQy9LQnYEo8dIli5YPxkjR0kHjsE2DBkgOW0f/y+\
f9LsdXdCMnTa//c+YAGDO0YM4O7yNe/tt/FPHl95pYUu1sXUG19/CIiKSHog7aeBPhU9sEXXdU/W\
Bydi03QdOXdgC9CXveZhe4bKgfrQ6h1blTn1q2nLu3p7IsBvGOhX2z5IvPOd54GzQEJkxKYpMq9k\
3FOxeserVNY9mkjES/bSxFhB13XKguXQF/1n/661G4CTQA/ZTVMjt835gEnArNDKTd8NzvzS6rTt\
mJaVwrK8WYMbC2iahmEEMAIBDJ8et6Ifvd2/Z8PvgDNAN9kpMJ8BkDEhDNT6qufMDi1+dllgeuOX\
jcraeb5AmanrOj7dh6br6L6czzkt33Gfrmf7+Qb7aZqGSOb1mUxTmYe0jpP9rXBUzufBftc/rqxk\
3O698Hmq/eiBxMGde1XX8dPABTLf/OCWsEIGQOaBSXnWiIEWAgwyoTKRIWQSXD+ZCx5oCXJ2jcL1\
DRiATmZzURmZPTY6t4YBDpl5PpVteROZiPB/dp1sFwOEwK8AAAAASUVORK5CYII="

char* info_proxy;
int info_port;
int (*set_events)(xmlChar* data);
/**
* Функция инициализации драйвера
*/
extern int msa_driver_init(msa_module*);
//extern int init_event_manager(int time);
    
#endif
